@@ -3,6 +3,8 @@
 # pylint: disable=too-many-locals
 # pylint: disable=no-member
 # pylint: disable=bare-except
+# pylint: disable=too-many-arguments
+# pylint: disable=too-many-return-statements
 
 """
 Mie scattering calculations for perfect spheres.
@@ -462,7 +464,53 @@ def _small_mie_S1_S2(m, x, mu):
     return [S1, S2]
 
 
-def mie_S1_S2(m, x, mu):
+def normalization_factor(a, b, x, norm_str):
+    """
+    Figure out scattering function normalization.
+
+    Args:
+        a: complex array of An coefficients
+        b: complex array of Bn coefficients
+        x: dimensionless sphere size
+        norm_str: string describing type of normalization
+
+    Returns:
+        scaling factor needed for scattering function
+    """
+    norm = norm_str.lower()
+
+    if norm in ['bohren']:
+        return 1 / 2
+
+    if norm in ['wiscombe']:
+        return 1
+
+    n = np.arange(1, len(a) + 1)
+    cn = 2.0 * n + 1.0
+    qext = 2 * np.sum(cn * (a.real + b.real)) / x**2
+
+    if norm in ['a', 'albedo']:
+        return np.sqrt(np.pi * x**2 * qext)
+
+    qsca = 2 * np.sum(cn * (np.abs(a)**2 + np.abs(b)**2)) / x**2
+
+    if norm in ['1', 'one', 'unity']:
+        return np.sqrt(qsca * np.pi * x**2)
+
+    if norm in ['four_pi', '4pi']:
+        return np.sqrt(qsca * x**2 / 4)
+
+    if norm in ['qsca', 'scattering_efficiency']:
+        return np.sqrt(np.pi * x**2)
+
+    if norm in ['qext', 'extinction_efficiency']:
+        return np.sqrt(qsca * np.pi * x**2 / qext)
+
+    raise ValueError("normalization must be one of 'albedo' (default), 'one'"
+                     "'4pi', 'qext', 'qsca', 'bohren', or 'wiscombe'")
+
+
+def mie_S1_S2(m, x, mu, norm='albedo'):
     """
     Calculate the scattering amplitude functions for spheres.
 
@@ -500,17 +548,15 @@ def mie_S1_S2(m, x, mu):
             pi_nm1 = ((2 * n + 1) * mu[k] * pi_nm1 - (n + 1) * pi_nm2) / n
             pi_nm2 = temp
 
-    # calculate norm = sqrt(pi * Qext * x**2)
-    n = np.arange(1, nstop + 1)
-    norm = np.sqrt(2 * np.pi * np.sum((2 * n + 1) * (a.real + b.real)))
+    normalization = normalization_factor(a, b, x, norm)
 
-    S1 /= norm
-    S2 /= norm
+    S1 /= normalization
+    S2 /= normalization
 
     return [S1, S2]
 
 
-def mie_cdf(m, x, num):
+def mie_cdf(m, x, num, norm='albedo'):
     """
     Create a CDF for unpolarized scattering uniformly spaced in cos(theta).
 
@@ -533,7 +579,7 @@ def mie_cdf(m, x, num):
         cdf: array of cumulative distribution function values
     """
     mu = np.linspace(-1, 1, num)
-    s1, s2 = mie_S1_S2(m, x, mu)
+    s1, s2 = mie_S1_S2(m, x, mu, norm)
 
     s = (np.abs(s1)**2 + np.abs(s2)**2) / 2
 
@@ -547,7 +593,7 @@ def mie_cdf(m, x, num):
     return mu, cdf
 
 
-def mie_mu_with_uniform_cdf(m, x, num):
+def mie_mu_with_uniform_cdf(m, x, num, norm='albedo'):
     """
     Create a CDF for unpolarized scattering for uniform CDF.
 
@@ -573,7 +619,7 @@ def mie_mu_with_uniform_cdf(m, x, num):
         cdf: array of cumulative distribution function values
     """
     big_num = 2000                  # large to work with x up to 10
-    big_mu, big_cdf = mie_cdf(m, x, big_num)
+    big_mu, big_cdf = mie_cdf(m, x, big_num, norm)
     mu = np.empty(num)
     cdf = np.empty(num)
 
@@ -631,7 +677,7 @@ def generate_mie_costheta(mu_cdf):
     return x
 
 
-def i_per(m, x, mu):
+def i_per(m, x, mu, norm='albedo'):
     """
     Return the scattered intensity in a plane normal to the incident light.
 
@@ -648,12 +694,12 @@ def i_per(m, x, mu):
     Returns
        The intensity at each angle in the array mu.  Units [1/sr]
     """
-    s1, _ = mie_S1_S2(m, x, mu)
+    s1, _ = mie_S1_S2(m, x, mu, norm)
     intensity = np.abs(s1)**2
     return intensity.astype('float')
 
 
-def i_par(m, x, mu):
+def i_par(m, x, mu, norm='albedo'):
     """
     Return the scattered intensity in a plane parallel to the incident light.
 
@@ -670,12 +716,12 @@ def i_par(m, x, mu):
     Returns
        The intensity at each angle in the array mu.  Units [1/sr]
     """
-    _, s2 = mie_S1_S2(m, x, mu)
+    _, s2 = mie_S1_S2(m, x, mu, norm)
     intensity = np.abs(s2)**2
     return intensity.astype('float')
 
 
-def i_unpolarized(m, x, mu):
+def i_unpolarized(m, x, mu, norm='albedo'):
     """
     Return the unpolarized scattered intensity at specified angles.
 
@@ -692,7 +738,7 @@ def i_unpolarized(m, x, mu):
     Returns
        The intensity at each angle in the array mu.  Units [1/sr]
     """
-    s1, s2 = mie_S1_S2(m, x, mu)
+    s1, s2 = mie_S1_S2(m, x, mu, norm)
     intensity = (np.abs(s1)**2 + np.abs(s2)**2) / 2
     return intensity.astype('float')
 
@@ -718,7 +764,7 @@ def ez_mie(m, d, lambda0, n_env=1.0):
     return mie(m_env, x_env)
 
 
-def ez_intensities(m, d, lambda0, mu, n_env=1.0):
+def ez_intensities(m, d, lambda0, mu, n_env=1.0, norm='albedo'):
     """
     Return the scattered intensities from a sphere.
 
@@ -744,7 +790,7 @@ def ez_intensities(m, d, lambda0, mu, n_env=1.0):
     m_env = m / n_env
     lambda_env = lambda0 / n_env
     x_env = np.pi * d / lambda_env
-    s1, s2 = mie_S1_S2(m_env, x_env, mu)
+    s1, s2 = mie_S1_S2(m_env, x_env, mu, norm)
     ipar = np.abs(s2)**2
     iper = np.abs(s1)**2
     Ipar = ipar.astype('float')
